{
  Desktop Sprite Demo 1.51

  By Nelson Chu 1998

  Email : eg_cshaa@stu.ust.hk
  Wpage : http://home.ust.hk/~eg_cshaa

  Description:
   It basically shows how to make sprite animation on the Win95 desktop
   like the toy programs "ScreenMate". The animation is done by copying
   rectangles directly to the desktop - not by creating a transparent
   window. With WinAPI functions like getforegroundwindow() or GetwindowRect(),
   you can make your character sitting on the foreground window.

  New in v1.51:
   1. Update the background if changed
   2. Avoid painting on the foreground window
   3. Clear traces when terminates
   4. Fixed bug of not releasing desktop DC in v1.5

  To-do list:
   1. Mouse draggable
   2. Popup menu
   3. Make it a component for easy production of programs like "ScreenMate"

  Compilation Note:
   Requires RXlib 2.4 or above. Please install it first.


  If you improve on it, did one of the items in the To-do list, or make
  something nice out of this demo, please send me a copy too! ;)
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Animate, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    AnimatedImage1: TAnimatedImage;
    Label1: TLabel;
    Label2: TLabel;
    Bevel1: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AnimatedImage1FrameChanged(Sender: TObject);
  private
    { Private declarations }
    nx, ny, ox, oy, dx, dy : integer; // new, old (x,y) and deviation
    sp_w : integer; // sprite width
    sp_h : integer; // sprite height
    sc   : TCanvas; // screen canvas
    working, savedback : TBitmap;
    TopTitle : PChar;    // for tracking which foreground window
    OldTopWindow : Hwnd; // for tracking if foreground window changed
    procedure SaveBackground(Sender:Tobject);
    procedure MoveSprite(Sender:Tobject);
    procedure PaintSprite(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.savebackground(Sender:Tobject);
begin
  sc:=TCanvas.Create;
  try
    sc.Handle:=GetDC(0);
    savedback.canvas.CopyRect( rect(0,0,sp_w, sp_h), sc,
                               rect(nx, ny, nx+sp_w, ny+sp_h));
    ReleaseDC(0, sc.handle);
  finally
    sc.free;
  end;
end;

function XslateRect(R:Trect; x,y:integer):Trect;
begin
 with R do Result:=Rect(left-x, top-y, right-x, bottom-y);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  sp_w:=animatedimage1.Width;
  sp_h:=animatedimage1.Height;

  try working:=Tbitmap.Create; except working.free; end;
  try savedback:=Tbitmap.Create; except savedback.free; end;
  try TopTitle:=strAlloc(20); except StrDispose(TopTitle); end;

  savedback.width:=sp_w;
  savedback.height:=sp_h;
  working.width:=sp_w;
  working.height:=sp_h;

  nx:=(screen.Width-sp_w) div 2;
  ny:=(screen.Height-sp_h) div 2;
  ox:=nx;  oy:=ny;
  savebackground(Self);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  // refresh the whole screen at the end
  RedrawWindow(0, nil, 0, RDW_ERASE or RDW_INVALIDATE or RDW_ALLCHILDREN);
  working.free;
  savedback.free;
  StrDispose(TopTitle);
end;

procedure TForm1.AnimatedImage1FrameChanged(Sender: TObject);
begin
  PaintSprite(self);
  MoveSprite(self);
end;


procedure TForm1.PaintSprite(Sender: TObject);
var TopRgn, newRgn, oldRgn, TempRgn    : Hrgn;
    TopRect, newRect, oldRect, OrgRect, TempRect : TRect;
    TopWindow : Hwnd;
begin
  newRect:=Rect(nx, ny, nx+sp_w, ny+sp_h);
  oldRect:=Rect(ox, oy, ox+sp_w, oy+sp_h);
  orgRect:=Rect(0, 0, sp_w, sp_h);

  TopWindow:=GetForegroundWindow;
  GetwindowRect(TopWindow,TopRect);
  GetwindowText(TopWindow, TopTitle, 19);
  TopRgn:=CreateRectRgnIndirect(TopRect);
  label1.caption:='Foreground Window: '+TopTitle;

  newRgn:=CreateRectRgnIndirect(newRect);
  oldRgn:=CreateRectRgnIndirect(oldRect);
  TempRgn:=CreateRectRgnIndirect(oldRect);

  try
  sc:=TCanvas.Create;
   try
    sc.handle:=getDC(0);

    // copy the new background to 'working'
    working.canvas.CopyRect(orgRect, sc, newRect);
    // recover the invalid background just copied
    CombineRgn(TempRgn, newRgn, oldRgn, RGN_AND);
    getRgnBox(TempRgn, TempRect);
    working.canvas.CopyRect(XslateRect(TempRect, nx, ny), savedback.canvas,
                            XslateRect(TempRect, ox, oy) );

    // grep the new background area
    savedback.canvas.CopyRect(orgRect, working.canvas, OrgRect);
    animatedimage1.DoPaintImageOn(working.canvas, 0, 0);
    // the following is for avoiding painting on the active form
    if IntersectRect(TempRect, TopRect, NewRect) and
      (TopTitle<>'Program Manager') // not when the wallpaper, etc is clicked
       then working.canvas.CopyRect(XslateRect(TempRect, nx, ny), sc, TempRect);
    // put the sprite on the screen
    sc.CopyRect(newRect, working.canvas, orgRect);

    ReleaseDC(0, sc.handle);
   finally
     sc.free;
   end;

   // erase the trace by updating the region
   CombineRgn(TempRgn, oldRgn, newRgn, RGN_DIFF);
   if TopTitle<>'Program Manager' then
       CombineRgn(TempRgn, TempRgn, TopRgn, RGN_DIFF);
   RedrawWindow(0, nil, TempRgn, RDW_ERASE or RDW_INVALIDATE or RDW_ALLCHILDREN);

   if oldTopwindow<>Topwindow then // foreground window changes
    RedrawWindow(0, nil, OldRgn, RDW_ERASE or RDW_INVALIDATE or RDW_ALLCHILDREN);

  finally
   DeleteObject(TopRgn);
   DeleteObject(TempRgn);
   DeleteObject(NewRgn);
   DeleteObject(OldRgn);
  end;

  OldTopWindow:=TopWindow;
end;


procedure TForm1.MoveSprite(Sender:Tobject);
// this is where you define your character's movement. You can use
// GetCursorPos() to play with the mouse pointer.
var pt:Tpoint;
begin
  getCursorPos(pt);
  if ny<pt.y then dy:=1 else
  if ny>pt.y then dy:=-1;
  dx:=4;

  ox:=nx;    oy:=ny;
  nx:=nx+dx; ny:=ny+dy;

  // wrap the sprite
  if nx<-sp_w then
    begin nx:=screen.Width-sp_w;
          savebackground(Self); end;
  if nx>screen.width then
    begin nx:=0;
          savebackground(Self); end;
end;

end.
